// Package `lib` provides a set of common types (mostly interfaces
// and function declarations) and some basic functions.
package lib

import (
	stdlib_context "context"
	"sync"
)

type ActionSpec interface {
	Handler() ActionHandler
	Receivers() []string
}

type ActionConfig interface {
	Pattern() string
	Specs() []ActionSpec
}

type Config interface {
	Name() string
	Starter() StartProc
	Step() StepProc
	MessageHandler() MessageHandler
	Actions() []ActionConfig
	Children() []Config
	Add(Config)
	WithAction(string, ActionSpec) Config
}

type Services map[string]Context

type ContextState interface{}

type Context interface {
	stdlib_context.Context
	Config() Config
	Parent() Context
	Services() Services
	ChildContext(Config) Context
	State() ContextState
	WithState(ContextState) Context
	Mailbox() chan Message
	HandleMsg(Message) bool
	WaitGroup() *sync.WaitGroup
	Stop()
}

type Address interface {
	Service() string
	Send(Context, Message)
}

type Message interface {
	Action() string
	Sender() Address
}

type Action interface {
	Context() Context
	Spec() ActionSpec
	Message() Message
}

type Proc = func(Context)
type StartProc = Proc
type StepProc = func(Context) bool
type MessageHandler = func(Context, Message) bool
type ActionHandler = func(Action) bool

// async Runners

func RunCtx(ctx Context, fct Proc) {
	ctx.WaitGroup().Add(1)
	go func() {
		defer ctx.WaitGroup().Done()
		fct(ctx)
	}()
}
